/**
 * OW2 Util
 * Copyright (C) 2008 SERLI
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 *
 * --------------------------------------------------------------------------
 * $Id UrlResourceFetcher.java $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.plan.fetcher.impl.url;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.net.URLConnection;

import org.ow2.util.file.FileUtils;
import org.ow2.util.file.FileUtilsException;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;
import org.ow2.util.plan.bindings.deploymentplan.url.UrlDeployment;
import org.ow2.util.plan.bindings.exceptions.InvalidDeploymentException;
import org.ow2.util.plan.bindings.exceptions.InvalidRepositoryException;
import org.ow2.util.plan.bindings.repository.Repository;
import org.ow2.util.plan.bindings.repository.RepositoryKind;
import org.ow2.util.plan.fetcher.api.IResourceFetcher;
import org.ow2.util.plan.fetcher.api.exceptions.FetcherException;
import org.ow2.util.plan.fetcher.api.exceptions.ResourceFetcherNotResolvedException;
import org.ow2.util.plan.fetcher.api.exceptions.ResourceNotFoundException;
import org.ow2.util.plan.fetcher.impl.AbsResourceFetcherImpl;


/**
 * Implementation of the IResourceFetcher interface for URL deployments.
 * @author Mickaël LEDUQUE
 */
public class UrlResourceFetcher extends AbsResourceFetcherImpl implements IResourceFetcher {

    /**
     * The logger.
     */
    private Log logger = LogFactory.getLog(UrlResourceFetcher.class);

    /**
     * The repository where the resource has been found.
     */
    private Repository foundRepository = null;

    /**
     * The rURL to the resource.
     */
    private URL foundURL = null;

    /**
     * The date of the last modification check.
     */
    private Long lastModifiedCheck = null;

    /**
     * The local copy of the resource.
     */
    private File localFile = null;

    /**
     * {@inheritDoc}
     */
    public void resolve() throws FetcherException {

        if (!(this.deployment instanceof UrlDeployment)) {
            throw new FetcherException(new InvalidDeploymentException("Incompatible class " + this.deployment.getClass()));
        }
        UrlDeployment urlDeployment = (UrlDeployment) this.deployment;

        logger.info("trying to resolve resource {0}", urlDeployment.getResource());

        /*
         * Try to get the resource from each repository in the repository
         * manager
         */
        localFile = null;
        for (Repository repository : this.getRepositoryManager()) {
            try {
                localFile = searchRepository(repository);
            } catch (FetcherException e) {
                logger.debug("resource {0} was not found on repository {1}", this.deployment, repository);
            }
            if (localFile != null) {
                // the resource was found, we can put an end to the search.
                return;
            }
        }
        if (localFile == null) {
            throw new FetcherException(new ResourceNotFoundException(urlDeployment.toString()));
        }
    }

    /*
     * Javadoc error ?
     * @throws ProtocolNotSupportedException if the protocol for the repository
     *         is not supported.
     */

    /**
     * Tries to find the requested deployment on the given repository.
     * @param repository the repoqitory on which the search will be done.
     * @return a local file for the resource.
     * @throws MalformedURLException if the URL of the repository is invalid.
     * @throws ResourceNotFoundException if the resource id not a the given URL.
     */
    private File searchRepository(final Repository repository) throws FetcherException {
        URL repositoryURL = null;
        try {
            repositoryURL = new URL(repository.getUrl());
        } catch (MalformedURLException e) {
            throw new FetcherException(new InvalidRepositoryException("invalid URL", e));
        }
        URI baseURI = null;
        try {
            baseURI = repositoryURL.toURI();
        } catch (URISyntaxException e) {
            logger.error("Bad URI syntax in {0}", repositoryURL);
            throw new FetcherException(new InvalidRepositoryException("invalid URL", e));
        }
        URI resultURI = baseURI.resolve(((UrlDeployment) this.deployment).getResource());

        URL resultURL = null;
        try {
            resultURL = resultURI.toURL();
        } catch (MalformedURLException e) {
            logger.error("Bad URI syntax in {0}", resultURL);
            throw new FetcherException(new InvalidDeploymentException("invalid path to resource", e));
        }
        lastModifiedCheck = System.currentTimeMillis();
        InputStream input = null;
        try {
            input = resultURL.openStream();
        } catch (IOException e) {
            logger.error("Exception while opening connection to resource {0}", resultURL);
            throw new FetcherException("Exception while opening connection", e);
        }

        // found - set the repository
        this.foundRepository = repository;
        this.foundURL = resultURL;

        // create a copy of the resource.
        localFile = makeLocalFilePath(this.localRepositoriesBaseDir,
                                      this.foundRepository.getId(),
                                      ((UrlDeployment) this.deployment).getResource());
        // Create the local file path.
        if (localFile != null) {
            if (!localFile.getParentFile().exists()) {
                localFile.getParentFile().mkdirs();
            }
        }

        try {
            FileUtils.dump(input, localFile);
        } catch (FileUtilsException e) {
            logger.error("Exception while copying data from resource {0};exception: {1}", resultURL, e);
            throw new FetcherException("Exception while copying resource data", e);
        }

        return localFile;
    }

    /**
     * {@inheritDoc}
     */
    public RepositoryKind getSupportedRepositoryType() {
        return RepositoryKind.URL;
    }

    /**
     * {@inheritDoc}
     */
    public File getResource() throws ResourceFetcherNotResolvedException {
        if (this.localFile == null) {
            throw new ResourceFetcherNotResolvedException();
        }
        return this.localFile;
    }

    /**
     * {@inheritDoc}
     */
    public boolean hasChanged() throws FetcherException {
        URLConnection connection = null;

        try {
            connection = this.foundURL.openConnection();
        } catch (IOException e) {
            throw new FetcherException(e);
        }
        long time = System.currentTimeMillis() - 1;
        long changed = connection.getLastModified();
        if (this.lastModifiedCheck < changed) {
            // The resource has changed
            return true;
        } else {
            this.lastModifiedCheck = time;
            return false;
        }

    }

    /**
     * {@inheritDoc}
     */
    public void update() throws FetcherException {
        this.searchRepository(this.foundRepository);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder(super.toString());
        sb.append(":useRepository=");
        if (this.foundRepository != null) {
            sb.append(this.foundRepository);
        } else {
            sb.append("null");
        }
        sb.append(":resolved=");
        sb.append(localFile != null);
        sb.append(":lastCheck=");
        sb.append(this.lastModifiedCheck);
        return sb.toString();
    }
}
